==> flink-table-common-1.12.0-sources.jar: org/apache/flink/table/types/inference/strategies/CompositeArgumentTypeStrategy.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.flink.table.types.inference.strategies;

import org.apache.flink.annotation.Internal;
import org.apache.flink.table.functions.FunctionDefinition;
import org.apache.flink.table.types.DataType;
import org.apache.flink.table.types.inference.ArgumentTypeStrategy;
import org.apache.flink.table.types.inference.CallContext;
import org.apache.flink.table.types.inference.Signature;
import org.apache.flink.table.types.logical.utils.LogicalTypeChecks;

import java.util.Optional;

/**
 * Strategy that checks that the argument has a composite type.
 */
@Internal
public class CompositeArgumentTypeStrategy implements ArgumentTypeStrategy {
	@Override
	public Optional<DataType> inferArgumentType(
			CallContext callContext,
			int argumentPos,
			boolean throwOnFailure) {
		DataType dataType = callContext.getArgumentDataTypes().get(argumentPos);
		if (!LogicalTypeChecks.isCompositeType(dataType.getLogicalType())) {
			if (throwOnFailure) {
				throw callContext.newValidationError(
					"A composite type expected. Got: %s",
					dataType);
			}
			return Optional.empty();
		}

		return Optional.of(dataType);
	}

	@Override
	public Signature.Argument getExpectedArgument(
		FunctionDefinition functionDefinition,
		int argumentPos) {
		return Signature.Argument.of("<COMPOSITE>");
	}
}

==> flink-table-common-1.15.0-sources.jar: org/apache/flink/table/types/inference/strategies/CompositeArgumentTypeStrategy.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.flink.table.types.inference.strategies;

import org.apache.flink.annotation.Internal;
import org.apache.flink.table.functions.FunctionDefinition;
import org.apache.flink.table.types.DataType;
import org.apache.flink.table.types.inference.ArgumentTypeStrategy;
import org.apache.flink.table.types.inference.CallContext;
import org.apache.flink.table.types.inference.Signature;
import org.apache.flink.table.types.logical.utils.LogicalTypeChecks;

import java.util.Optional;

/** Strategy that checks that the argument has a composite type. */
@Internal
public class CompositeArgumentTypeStrategy implements ArgumentTypeStrategy {
    @Override
    public Optional<DataType> inferArgumentType(
            CallContext callContext, int argumentPos, boolean throwOnFailure) {
        DataType dataType = callContext.getArgumentDataTypes().get(argumentPos);
        if (!LogicalTypeChecks.isCompositeType(dataType.getLogicalType())) {
            if (throwOnFailure) {
                throw callContext.newValidationError(
                        "A composite type expected. Got: %s", dataType);
            }
            return Optional.empty();
        }

        return Optional.of(dataType);
    }

    @Override
    public Signature.Argument getExpectedArgument(
            FunctionDefinition functionDefinition, int argumentPos) {
        return Signature.Argument.of("<COMPOSITE>");
    }
}
